/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.crov.prase.controller.prase;

import com.arrebol.taxiservicios.controller.connection.ConnectionManager;
import com.arrebol.taxiservicios.controller.util.HibernateUtil;
import com.arrebol.taxiservicios.model.catalog.Location;
import com.arrebol.taxiservicios.model.core.User;
import com.arrebol.taxiservicios.model.enums.GenericEnumType;
import com.crov.prase.model.prase.PagosPoliza;
import com.crov.prase.model.prase.Poliza;
import java.io.Serializable;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 *
 * @author Crov
 */
public class PagosPolizaController extends ConnectionManager implements Serializable {

    private static final long serialVersionUID = -8772793021819350069L;
    final Logger logger = LogManager.getLogger(getClass());

    public List<PagosPoliza> fillPagosPolizaByPoliza(Poliza poliza) {
        logger.info("fillPagosPolizaByPoliza");
        List<PagosPoliza> resultList = null;
        Transaction transaction = null;
        try {
            logger.info("Searching pagos for poliza..");
            Session session = HibernateUtil.getSessionFactory().getCurrentSession();
            transaction = session.beginTransaction();
            CriteriaBuilder builder = session.getCriteriaBuilder();

            CriteriaQuery<PagosPoliza> criteria = builder.createQuery(PagosPoliza.class);
            Root<PagosPoliza> root = criteria.from(PagosPoliza.class);

            Predicate criterio1 = builder.equal(root.get("poliza"), poliza);
            Predicate criterio2 = builder.equal(root.get("estatusActivo"), GenericEnumType.ENABLED);
            criteria.where(builder.and(criterio1, criterio2));
            criteria.orderBy(builder.asc(root.get("fechaAPagar")));

            resultList = session.createQuery(criteria).getResultList();

            logger.info("Total of pagos found: " + (null == resultList ? -1 : resultList.size()));
            transaction.commit();
        } catch (HibernateException e) {
            logger.error("Pagos cannot be loaded ");
            rollback(transaction);
        } catch (Exception e) {
            logger.error("Method fillPagosPolizaByPoliza ", e);
            rollback(transaction);
        }
        return resultList;
    }

    public List<PagosPoliza> fillPagosPolizaFechaAPagarByLocationFechaInicioAndFechaFin(Location location, Date startDate, Date endDate) {
        logger.info("fillPagosPolizaByPoliza");
        List<PagosPoliza> resultList = null;
        Transaction transaction = null;
        try {
            logger.info("Searching pagos for poliza..");
            Session session = HibernateUtil.getSessionFactory().getCurrentSession();
            transaction = session.beginTransaction();
            CriteriaBuilder builder = session.getCriteriaBuilder();

            CriteriaQuery<PagosPoliza> criteria = builder.createQuery(PagosPoliza.class);
            Root<PagosPoliza> root = criteria.from(PagosPoliza.class);

            LocalDate localStartDate = startDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
            LocalDate localEndDate = endDate.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();

            Predicate criterio1 = builder.equal(root.get("poliza").get("location"), location);
            Predicate criterio2 = builder.equal(root.get("estatusActivo"), GenericEnumType.ENABLED);
            Predicate criterio3 = builder.equal(root.get("pagoEstatus"), GenericEnumType.DISABLED);
            Predicate startDatePredicate = builder.greaterThanOrEqualTo(root.get("fechaAPagar").as(LocalDate.class), localStartDate);
            Predicate endDatePredicate = builder.lessThanOrEqualTo(root.get("fechaAPagar").as(LocalDate.class), localEndDate);
            criteria.where(builder.and(criterio1, criterio2,criterio3,startDatePredicate, endDatePredicate));
            criteria.orderBy(builder.asc(root.get("fechaAPagar")));

            resultList = session.createQuery(criteria).getResultList();

            logger.info("Total of pagos found: " + (null == resultList ? -1 : resultList.size()));
            transaction.commit();
        } catch (HibernateException e) {
            logger.error("Pagos cannot be loaded ");
            rollback(transaction);
        } catch (Exception e) {
            logger.error("Method fillPagosPolizaByPoliza ", e);
            rollback(transaction);
        }
        return resultList;
    }

    public PagosPoliza fillFoliosByPagos(String abreviacionFolio) {
        PagosPoliza result = null;
        Transaction transaction = null;
        try {
            logger.info("Search last generated folio..");

            Session session = HibernateUtil.getSessionFactory().getCurrentSession();
            transaction = session.beginTransaction();

            String query = "SELECT tap FROM PagosPoliza tap "
                    + "WHERE tap.folio LIKE '" + abreviacionFolio + "%' "
                    + "ORDER BY "
                    + "SUBSTRING_INDEX(tap.folio, '-', 1) DESC, "
                    + "SUBSTRING_INDEX(SUBSTRING_INDEX(tap.folio, '-', -2), '-', -6) DESC ";

            result = (PagosPoliza) session.createQuery(query).setMaxResults(1).uniqueResult();
            transaction.commit();
        } catch (HibernateException e) {
            logger.error("Search for the last generated folio " + e);
            rollback(transaction);
        } catch (Exception e) {
            logger.error("Method fillFoliosByPagos() " + e);
            rollback(transaction);
        }
        return result;
    }
    
    public boolean updatePago(PagosPoliza pago, User user) {
        logger.info("addPagos");
        boolean success = false;
        Transaction transaction = null;

        try {
            Session session = HibernateUtil.getSessionFactory().getCurrentSession();
            transaction = session.beginTransaction();

            pago.setLastUpdatedOn(correcciónDeHr(new Date(), -7));
            pago.setLastUpdatedBy(user);
            session.update(pago);
            transaction.commit();

            success = true;
        } catch (HibernateException e) {
            logger.error("Venta can not be added ", e);
            rollback(transaction);
        } catch (Exception e) {
            logger.error("Method addVenta() ", e);
            rollback(transaction);
        }
        return success;
    }

    private Date correcciónDeHr(Date fecha, int horas) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(fecha);
        calendar.add(Calendar.HOUR_OF_DAY, horas);
        Date nuevaFecha = calendar.getTime();
        return nuevaFecha;
    }

}
