/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.crov.prase.prase;

import com.arrebol.taxiservicios.beans.TaxiGenericBean;
import com.arrebol.taxiservicios.controller.admin.GenericValidationController;
import com.arrebol.taxiservicios.model.enums.GenericEnumType;
import com.arrebol.taxiservicios.model.enums.MetodoPago;
import com.arrebol.taxiservicios.model.enums.PolizaEstatus;
import com.crov.prase.controller.prase.PagosPolizaController;
import com.crov.prase.controller.prase.PolizaController;
import com.crov.prase.model.prase.PagosPoliza;
import com.crov.prase.model.prase.Poliza;
import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.view.ViewScoped;
import javax.inject.Named;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 *
 * @author Crov
 */
@Named("pagosPolizasBean")
@ViewScoped
public class PagosPolizasBean extends TaxiGenericBean implements Serializable {

    private static final long serialVersionUID = -136031528440947006L;
    final Logger logger = LogManager.getLogger(PolizaBean.class);

    private PolizaController polizaController;
    private PagosPolizaController pagosPolizaController;
    private GenericValidationController genericController;

    private List<Poliza> poliza;
    private Poliza selectedPoliza;
    private List<PagosPoliza> listPagosPolizas;
    private PagosPoliza selectedPago;
    private String pagoSiguiente;
    private Double cantidadAPagar;
    private Boolean pagoExitoso;
    private String imagen;
    private String metodoPago;

    @PostConstruct
    public void init() {
        logger.info("init");
        try {
            loadBundlePropertyFile();
            setPolizaController(new PolizaController());
            setPagosPolizaController(new PagosPolizaController());
            setGenericController(new GenericValidationController());
            cargarPolizas();
        } catch (Exception e) {
            logger.error(e);
        }
    }

    public void cargarPolizas() {
        logger.info("cargarPolizas()");
        setPoliza(getPolizaController().findActive(getLoggedUser().getLocation()));
    }

    public void cargarPagosPoliza() {
        logger.info("cargarPagosPoliza()");
        try {
            // se caga la lista de pagos 
            setListPagosPolizas(getPagosPolizaController().fillPagosPolizaByPoliza(getSelectedPoliza()));
            // se encuentra el siguiente pago a realizar 
            String idSiguinetePago = "";
            for (PagosPoliza pa : getListPagosPolizas()) {
                if (pa.getPagoEstatus().equals(GenericEnumType.DISABLED)) {
                    idSiguinetePago = pa.getId();
                    break;
                }
            }
            setPagoSiguiente(idSiguinetePago);
        } catch (Exception e) {
            logger.error("cargarPagosPoliza(): " + e);
        }
    }

    public void cargarPago() {
        setCantidadAPagar(getSelectedPago().getPago());
    }

    public void pagar() {
        logger.info("pagar()");
        try {

            setPagoExitoso(false);

            if (getGenericController().existClosingDayByCreatedOn(new Date(), getLoggedUser())) {
                showMessage(FacesMessage.SEVERITY_WARN, "Error", "No se puede hacer ninguna acción si existe un corte del día");
                return;
            }

            if (getGenericController().existStableGeneralBoxByCurdate(getLoggedUser())) {
                showMessage(FacesMessage.SEVERITY_WARN, "Dia cerrado", "No se puede hacer ninguna acción porque ya existe un cuadre de caja chica del día");
                return;
            }

            // se valida el pago realizado sea el siguiente según el consecutivo
            if (getSelectedPago().getId().equals(getPagoSiguiente())) {
                setImagen("images/prase_logo.jpeg");

                // se optiene el folio consecutivo
                String folio = obtenerFolio("A-PRASE");
                if (folio != null) {
                    // se actualiza el pago
                    getSelectedPago().setPagoEstatus(GenericEnumType.ENABLED);
                    getSelectedPago().setFolio(folio);
                    getSelectedPago().setFechaPago(correcciónDeHr(new Date(), -7));
                    getSelectedPago().setCobro(getLoggedUser());
                    getSelectedPago().setCantidadPagada(getSelectedPago().getPago());
                    getSelectedPago().setDiferenciaPago(0.0);
                    switch (getMetodoPago()) {
                        case "EFECTIVO":
                            getSelectedPago().setMetodoPago(MetodoPago.EFECTIVO);
                            break;
                        case "TARJETA":
                            getSelectedPago().setMetodoPago(MetodoPago.TARJETA);
                            break;
                        case "DEPOSITO":
                            getSelectedPago().setMetodoPago(MetodoPago.DEPOSITO);
                            break;
                        case "TRASFERENCIA":
                            getSelectedPago().setMetodoPago(MetodoPago.TRASFERENCIA);
                            break;
                        default:
                            getSelectedPago().setMetodoPago(MetodoPago.EFECTIVO);
                    }
                    getPagosPolizaController().updatePago(getSelectedPago(), getLoggedUser());

                    // se actuliza la venta                     
                    getSelectedPoliza().setCantidadPagada(getSelectedPoliza().getCantidadPagada() + getSelectedPago().getPago());
                    getSelectedPoliza().setLastUpdatedBy(getLoggedUser());
                    getSelectedPoliza().setLastUpdatedOn(new Date());
                    //se valida si la venta ya fue liquidada                        
                    double cantidadPa = getSelectedPoliza().getCantidadPagada();
                    double costoTotal = getSelectedPoliza().getAmount();
                    double tolerancia = 0.001;
                    if (Math.abs(cantidadPa - costoTotal) < tolerancia) {
                        getSelectedPoliza().setEstatus(PolizaEstatus.LIQUIDADO);
                    }
                    getPolizaController().update(getSelectedPoliza());

                    // se actualiza la lista de pagos y polizas 
                    cargarPagosPoliza();
                    cargarPolizas();
                    setPagoExitoso(true);
                    logger.info("El pago se realizó correctamente");
                    showMessage(FacesMessage.SEVERITY_INFO, "Pago realizado", "El pago se realizó correctamente");
                } else {
                    logger.error("Ocurrió un error al generar el folio");
                    showMessage(FacesMessage.SEVERITY_INFO, "Error", "Ocurrió un error al generar el folio");
                }

            } else {
                showMessage(FacesMessage.SEVERITY_WARN, "Error", "Solo puede pagar el pago consecutivo");
            }

        } catch (Exception e) {
            showMessage(FacesMessage.SEVERITY_INFO, "Error", "Ocurrió un error al realizar el pago");
            logger.error("pagar(): " + e);
        }
    }

    private String obtenerFolio(String ab) {
        logger.info("Obteniendo último folio ");
        try {
            // Se cargan los abonos para validar el folio
            String abreviacion = ab;
            PagosPoliza ultimoFolio = getPagosPolizaController().fillFoliosByPagos(abreviacion);
            // Se valida si ya existen folios registrados para el proyecto y se le asigna el último encontrado
            int consecutivo = 0;
            if (ultimoFolio != null) {
                String folior = ultimoFolio.getFolio();
                // Dividir el folio en partes utilizando el guion como separador
                String[] partes = folior.split("-");
                // Obtener la parte del consecutivo
                String consecutivoParte = partes[2];
                // Convertir el consecutivo a un entero
                consecutivo = Integer.parseInt(consecutivoParte);
            }
            // se formatea el consecutivo                                         
            String numeroFormateado = String.format("%06d", consecutivo + 1);
            String folio;
            // Se obtiene el año actual
            Calendar calendario = Calendar.getInstance();
            int ano = calendario.get(Calendar.YEAR);
            String anoActual = String.valueOf(ano);
            // Se valida si se creara el primer folio 
            if (consecutivo == 0) {
                folio = abreviacion + "-" + "000001" + "-" + anoActual;
            } else {
                folio = abreviacion + "-" + numeroFormateado + "-" + anoActual;
            }
            return folio;
        } catch (NumberFormatException e) {
            logger.error("Ocurrió un erro al obtener el último folio ");
            showMessage(FacesMessage.SEVERITY_ERROR, "Error", "Ocurrió un error al generar el folio");
            return null;
        }
    }

    public void reimprimirTicket() {
        setPagoExitoso(true);
        try {
            setImagen("images/prase_logo.jpeg");
        } catch (Exception e) {
            logger.error("Error al definir el logo para el tiked  " + e);
        }
    }

    private Date correcciónDeHr(Date fecha, int horas) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(fecha);
        calendar.add(Calendar.HOUR_OF_DAY, horas);
        Date nuevaFecha = calendar.getTime();
        return nuevaFecha;
    }

    public String validarColores(String estatus) {
        // Validadar color 
        if (estatus.equals("LIQUIDADO")) {
            return "bgVerde";
        } else {
            return "";
        }
    }

    public PolizaController getPolizaController() {
        return polizaController;
    }

    public void setPolizaController(PolizaController polizaController) {
        this.polizaController = polizaController;
    }

    public List<Poliza> getPoliza() {
        return poliza;
    }

    public void setPoliza(List<Poliza> poliza) {
        this.poliza = poliza;
    }

    public Poliza getSelectedPoliza() {
        return selectedPoliza;
    }

    public void setSelectedPoliza(Poliza selectedPoliza) {
        this.selectedPoliza = selectedPoliza;
    }

    public List<PagosPoliza> getListPagosPolizas() {
        return listPagosPolizas;
    }

    public void setListPagosPolizas(List<PagosPoliza> listPagosPolizas) {
        this.listPagosPolizas = listPagosPolizas;
    }

    public PagosPoliza getSelectedPago() {
        return selectedPago;
    }

    public void setSelectedPago(PagosPoliza selectedPago) {
        this.selectedPago = selectedPago;
    }

    public PagosPolizaController getPagosPolizaController() {
        return pagosPolizaController;
    }

    public void setPagosPolizaController(PagosPolizaController pagosPolizaController) {
        this.pagosPolizaController = pagosPolizaController;
    }

    public String getPagoSiguiente() {
        return pagoSiguiente;
    }

    public void setPagoSiguiente(String pagoSiguiente) {
        this.pagoSiguiente = pagoSiguiente;
    }

    public Double getCantidadAPagar() {
        return cantidadAPagar;
    }

    public void setCantidadAPagar(Double cantidadAPagar) {
        this.cantidadAPagar = cantidadAPagar;
    }

    public GenericValidationController getGenericController() {
        return genericController;
    }

    public void setGenericController(GenericValidationController genericController) {
        this.genericController = genericController;
    }

    public Boolean getPagoExitoso() {
        return pagoExitoso;
    }

    public void setPagoExitoso(Boolean pagoExitoso) {
        this.pagoExitoso = pagoExitoso;
    }

    public String getImagen() {
        return imagen;
    }

    public void setImagen(String imagen) {
        this.imagen = imagen;
    }

    public String getMetodoPago() {
        return metodoPago;
    }

    public void setMetodoPago(String metodoPago) {
        this.metodoPago = metodoPago;
    }

}
